@@ -1,4 +1,4 @@
-# $Id: heatmap.2.R 967 2006-06-26 21:08:21Z nj7w $
+# $Id: heatmap.2.R 1252 2008-04-07 16:47:30Z warnes $
 
 heatmap.2 <- function (x,
 
@@ -62,6 +62,11 @@ heatmap.2 <- function (x,
                      xlab = NULL,
                      ylab = NULL,
 
+                     # plot layout
+                     lmat = NULL,
+                     lhei = NULL,
+                     lwid = NULL,
+
                      # extras
                      ...
                      )
@@ -270,22 +275,44 @@ else
     x[] <- ifelse(x<min.breaks, min.breaks, x)
     x[] <- ifelse(x>max.breaks, max.breaks, x)
 
+
+
+  
+
+  
     ## Calculate the plot layout
-    lmat <- rbind(4:3, 2:1)
-    lhei <- lwid <- c(keysize, 4)
-    if(!missing(ColSideColors)) { ## add middle row to layout
-	if(!is.character(ColSideColors) || length(ColSideColors) != nc)
-	    stop("'ColSideColors' must be a character vector of length ncol(x)")
-	lmat <- rbind(lmat[1,]+1, c(NA,1), lmat[2,]+1)
-	lhei <- c(lhei[1], 0.2, lhei[2])
-    }
-    if(!missing(RowSideColors)) { ## add middle column to layout
-	if(!is.character(RowSideColors) || length(RowSideColors) != nr)
-	    stop("'RowSideColors' must be a character vector of length nrow(x)")
-	lmat <- cbind(lmat[,1]+1, c(rep(NA, nrow(lmat)-1), 1), lmat[,2]+1)
-	lwid <- c(lwid[1], 0.2, lwid[2])
-    }
-    lmat[is.na(lmat)] <- 0
+    if( missing(lhei) || is.null(lhei) )
+      lhei <- c(keysize, 4)
+
+    if( missing(lwid) || is.null(lwid) )
+       lwid <- c(keysize, 4)
+
+    if( missing(lmat) || is.null(lmat) )
+       {
+         lmat <- rbind(4:3, 2:1)
+         
+         if(!missing(ColSideColors)) { ## add middle row to layout
+           if(!is.character(ColSideColors) || length(ColSideColors) != nc)
+             stop("'ColSideColors' must be a character vector of length ncol(x)")
+           lmat <- rbind(lmat[1,]+1, c(NA,1), lmat[2,]+1)
+           lhei <- c(lhei[1], 0.2, lhei[2])
+         }
+
+         if(!missing(RowSideColors)) { ## add middle column to layout
+           if(!is.character(RowSideColors) || length(RowSideColors) != nr)
+             stop("'RowSideColors' must be a character vector of length nrow(x)")
+           lmat <- cbind(lmat[,1]+1, c(rep(NA, nrow(lmat)-1), 1), lmat[,2]+1)
+           lwid <- c(lwid[1], 0.2, lwid[2])
+         }
+
+         lmat[is.na(lmat)] <- 0
+       }
+       
+     if(length(lhei) != nrow(lmat))
+       stop("lhei must have length = nrow(lmat) = ", nrow(lmat))
+
+     if(length(lwid) != ncol(lmat))
+       stop("lwid must have length = ncol(lmat) =", ncol(lmat))
 
     ## Graphics `output' -----------------------
 
@@ -348,7 +375,7 @@ else
     if(!missing(rowsep))
       for(rsep in rowsep)
         rect(xleft =0,          ybottom= (ncol(x)+1-rsep)-0.5,
-             xright=ncol(x)+1,  ytop   = (ncol(x)+1-rsep)-0.5 - sepwidth[2],
+             xright=nrow(x)+1,  ytop   = (ncol(x)+1-rsep)-0.5 - sepwidth[2],
              lty=1, lwd=1, col=sepcolor, border=sepcolor)
 
   
